//SFind v1.0 - Copyright(c) 1998 by JD Glaser - NT OBJECTives, Inc. ALL RIGHTS RESERVED
//NTFS Alternate Data Stream Finder

 ********************************************************************
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 1, or (at your option)
 *  any later version. *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details. *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

#include <windows.h>
#include <tchar.h>
#include <iomanip.h>
#include <stdio.h>

const int MAXPATH = 500;

void ScanDir(bool bScanSubs);
void PrintHelp();
void EnableBCKprivs();
void DumpHeader(WIN32_STREAM_ID& streamID, int iCount, char* pszFileName, int& iFileHits);
void ListStreams(char* pszFileName, int& iFileHits);

void main(int argc, char* argv[])
{
	bool bScanSubs = true;

	char CurDir[MAXPATH];

	// check if running on Windows NT, if not, display notice and terminate 
    if( GetVersion() & 0x80000000 ) 
    { 
      printf("SFind only runs on Windows NT...");
      return; 
    }

	//set thread priority to lowest
	HANDLE hThread;
	hThread = ::GetCurrentThread();
	::SetThreadPriority(hThread, THREAD_PRIORITY_LOWEST);

	if(argc > 1)
	{
		for (int ctr = 1; ctr < argc; ctr++)
		{
			const char* pArg = argv[ctr];

			if((0 == strcmp(pArg, "-ns")) || (0 == strcmp(pArg, "/ns")))
			{
				bScanSubs = false;
			}
			
			else if((0 == strcmp(pArg, "-?")) || (0 == strcmp(pArg, "/?")))
			{
				PrintHelp();
				return;
			}

			else
			{
				char* pNewPath = argv[ctr];
				::SetCurrentDirectory(pNewPath);
			}
		}
	}
	else
	{
		::GetCurrentDirectory(sizeof(CurDir), CurDir);
	}	

try
{
	//give privs to scan files to which you have no permissions
	EnableBCKprivs();

	printf("Searching...\n");
	//Start the recursive search
	ScanDir(bScanSubs);

	//reset to original starting point
	::SetCurrentDirectory(CurDir);

	//print a closing notice
	char* pEndNotice = "Finished";
	printf("\r%-79s", pEndNotice);
	return;
}

	catch(...)
	{
		printf("\nError - Shutting Down...");
		return;
	}
}

void ScanDir(bool bScanSubs)
{
	bool bPrintOnce = true;
	int iFileHits = 0;

	WIN32_FIND_DATA data;
	HANDLE hFile = ::FindFirstFile("*.*", &data);
	while(FindNextFile(hFile, &data))
	{
		if(data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY
			&& data.cFileName[0] != '.' && bScanSubs)
		{
			::SetCurrentDirectory(data.cFileName);
			bPrintOnce = true;

			//scan into the next sub-directory
			ScanDir(bScanSubs);
			::SetCurrentDirectory("..");
		}
	
		else
		{
			//Group files by directory
			if(bPrintOnce)
			{
				char CurDir[MAXPATH];
				::GetCurrentDirectory(sizeof(CurDir), CurDir);

				//zero out the last char to set a limit on the output
				CurDir[79] = '\0';
				printf("\r%-79s", CurDir);
				bPrintOnce = false;
				iFileHits = 0;
			}

			//Call list streams with current filename
			ListStreams(data.cFileName, iFileHits);
		}
	}
	//Close the handle
	::FindClose(hFile);
};

void ListStreams(char* pszFileName, int& iFileHits)
{
	//thanks to felix for doin the hard part, and for the even better zw way

  	HANDLE hFile;	
	LPCTSTR lpFileName = (LPCTSTR)pszFileName;

	hFile = CreateFile(lpFileName, GENERIC_READ, FILE_SHARE_WRITE, 0, OPEN_EXISTING, 0, 0);

	if (hFile == INVALID_HANDLE_VALUE || hFile == NULL)	
	{ 
	    DWORD error = GetLastError();	
		return; 
	}
	
	BYTE byteBuff[4096];
	DWORD dwRead;
	void* pContext = NULL;
	WIN32_STREAM_ID& streamID = *((WIN32_STREAM_ID*)byteBuff);

	int iCount = 0;
	while (1)
	{	// grab stream header
		if (!BackupRead(hFile, byteBuff, 20, &dwRead, FALSE, TRUE, &pContext))	
		{
			//ERROR HANDLER GOES HERE
		}
		if (dwRead == 0 )
			break;
		if (streamID.dwStreamNameSize > 0)	
		{
			if (!BackupRead(hFile, byteBuff + 20, streamID.dwStreamNameSize, &dwRead, FALSE, TRUE, &pContext))
			{ 
				//ERROR HANDLER GOES HERE
			}
			if (dwRead != streamID.dwStreamNameSize)
				break;
		}	
		
		iCount++;
		DumpHeader(streamID, iCount, pszFileName, iFileHits);
		// skip stream data	
		if(streamID.Size.QuadPart > 0)
		{	
			DWORD dwLo, dwHi;
			BackupSeek(hFile, 0xffffffffL, 0x7fffffffL, &dwLo, &dwHi, &pContext);	
		}
	}

	//Release 
	BackupRead(hFile, byteBuff, 0, &dwRead, TRUE, FALSE, &pContext);
	CloseHandle(hFile);
	return;
}

void DumpHeader(WIN32_STREAM_ID& streamID, int iCount, char* pszFileName, int& iFileHits)
{
	const char* pStreamName;

	switch(streamID.dwStreamId)
	{
	case BACKUP_ALTERNATE_DATA:	
		{
			char hiddenName[500];
			// Convert the Unicode full name to ANSI.
			WideCharToMultiByte( CP_ACP, 0, streamID.cStreamName, -1,
								hiddenName, 500, NULL, NULL ); 

			char seps[] = ":";;
			char* pToken;
			pToken = strtok(hiddenName, seps);
			if(iFileHits < 1)
				printf("\n");
			printf("  %s:%s Size: %I64d\n", pszFileName, pToken, streamID.Size.QuadPart);
			iFileHits++;
			pStreamName = "Alternate Stream";
		}
		break;
	default:
		break;
	}

}

void EnableBCKprivs()
{	
	HANDLE hToken;
	BYTE byteBuff[sizeof TOKEN_PRIVILEGES* 2];
	TOKEN_PRIVILEGES& tkp = *((TOKEN_PRIVILEGES*)byteBuff);

	OpenProcessToken( GetCurrentProcess(),
		TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hToken); 
	// set up SeBackupPrivilege, SeRestorePrivilege
	LookupPrivilegeValue(NULL, SE_BACKUP_NAME, &tkp.Privileges[0].Luid);

	LookupPrivilegeValue(NULL, SE_RESTORE_NAME, &tkp.Privileges[1].Luid);

	tkp.PrivilegeCount = 2;
	tkp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
	tkp.Privileges[1].Attributes = SE_PRIVILEGE_ENABLED;
	AdjustTokenPrivileges(hToken, FALSE, &tkp, sizeof tkp, NULL, NULL);
}

void PrintHelp()
{
	cout << "SFind v1.2.2 - Copyright(c) 1998, NT OBJECTives, Inc." << endl;
	cout << "Alternate Data Stream Finder" << endl;
	cout << "Programming by JD Glaser - All Rights Reserved" << endl;
	cout << "\t" << "Usage - sfind [path] /ns" << endl;
	cout << "\t" << "[dirpath]" << "\t" << "Directory to search - none equals current" << endl;
	cout << "\t" << "-ns" << "\t\t" << "Skip sub-directories" << endl;
	cout << "\t" << "- or /" << "\t\t" << "Either switch statement can be used" << endl;
	cout << "\t" << "-?" << "\t\t" << "Help" << endl;
	cout << "COMMAND PROMPT MUST HAVE A MINIMUM WIDTH OF 80 CHARACTERS" << endl;
	cout << "See http://www.ntobjectives.com for updates/fixes" << endl;
	return;
}