//AFind v1.2.1 - Copyright(c) 1998 by JD Glaser - NT OBJECTives, Inc. ALL RIGHTS RESERVED
//NTFS last access time finder

 ********************************************************************
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 1, or (at your option)
 *  any later version. *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details. *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 


#include <windows.h>
#include <tchar.h>
#include <iomanip.h>
#include <stdio.h>
#include <time.h>

void ScanDir(bool bScanSubs, char* pFileName, FILETIME ftBegSrchTime, FILETIME ftEndSrchTime);
void TimetToFileTime(time_t tTime, LPFILETIME pFileTime);
void DetermineOffsetTime(int iTimeUnit, char* pTimeFrame, long& lBegTimeTotal, long& lEndTimeTotal);
void DetermineActualTime(int iTimeUnit, char* pTimeFrame, FILETIME& ftBegTime, FILETIME& ftEndTime);
void PrintHelp();

const int MAXPATH = 500;
const int SECOND = 1;
const int MINUTE = 60;
const int HOUR = 3600;
const int DAY = 86400;

bool g_bTimeFrame = false;

void main(int argc, char* argv[])
{
	bool bScanSubs = true;
	bool bActualTime = false;
	char CurDir[MAXPATH];
	char* pFileName = NULL;
	long lBegSrchTime = 0;
	long lEndSrchTime = 0;
	time_t tCurrTime, tBegSearchTime, tEndSearchTime; 
	FILETIME ftBegAccessSrch, ftEndAccessSrch, ftBegSrchActual, ftEndSrchActual;
	SYSTEMTIME stBegLFAccess, stEndLFAccess;

	// check if running on Windows NT, if not, display notice and terminate 
    if( GetVersion() & 0x80000000 ) 
    { 
      printf("AFind only runs on Windows NT...");
      return; 
    }

try
{

	//Get the current time straight away
	time(&tCurrTime);

	//set thread priority to lowest
	HANDLE hThread;
	hThread = ::GetCurrentThread();
	::SetThreadPriority(hThread, THREAD_PRIORITY_LOWEST);

	::GetCurrentDirectory(sizeof(CurDir), CurDir);

	if(argc < 2)
	{
		PrintHelp();
		return;
	}

	if(argc >= 2)
	{
		for (int ctr = 1; ctr < argc; ctr++)
		{
			const char* pArg = argv[ctr];
			int iTimeUnit;

			if((0 == strcmp(pArg, "-ns")) || (0 == strcmp(pArg, "/ns")))
			{
				bScanSubs = false;
			}
			else if((0 == strcmp(pArg, "-f")) || (0 == strcmp(pArg, "/f")))
			{
				bScanSubs = false;
				pFileName = argv[ctr+1];

				WIN32_FIND_DATA data;
				//print the file access time and end execution, we're done
				HANDLE hFile = ::FindFirstFile(pFileName, &data);
				if(hFile != INVALID_HANDLE_VALUE)
				{
					FILETIME ftFileAccess;
					SYSTEMTIME stFileAccess;
					//Get the last-access time to local time.
					if(!FileTimeToLocalFileTime(&data.ftLastAccessTime, &ftFileAccess)) 
						return;

					//Convert the local file time from UTC to system time.
					if(!FileTimeToSystemTime(&ftFileAccess, &stFileAccess))
						return;
					TCHAR lpszString[1024];
					wsprintf(lpszString, "\n\t%s\t%02d/%02d/%d  %02d:%02d:%02d\n",
							data.cFileName,
							stFileAccess.wDay, stFileAccess.wMonth, stFileAccess.wYear,
							stFileAccess.wHour, stFileAccess.wMinute, stFileAccess.wSecond);
					printf(lpszString);
					return;
				}
				else
				{
					//exit 'cause no file
					char* pEndNotice = "No Such File";
					printf("\r%-79s", pEndNotice);
					return;
				}
				
			}
			else if((0 == strcmp(pArg, "-s")) || (0 == strcmp(pArg, "/s")))
			{
				iTimeUnit = SECOND;
				DetermineOffsetTime(iTimeUnit, argv[ctr+1], lBegSrchTime, lEndSrchTime);
				ctr++;
			}
			else if((0 == strcmp(pArg, "-m")) || (0 == strcmp(pArg, "/m")))
			{
				iTimeUnit = MINUTE;
				DetermineOffsetTime(iTimeUnit, argv[ctr+1], lBegSrchTime, lEndSrchTime);
				ctr++;
			}
			else if((0 == strcmp(pArg, "-h")) || (0 == strcmp(pArg, "/h")))
			{
				iTimeUnit = HOUR;
				DetermineOffsetTime(iTimeUnit, argv[ctr+1], lBegSrchTime, lEndSrchTime);
				ctr++;
			}
			else if((0 == strcmp(pArg, "-d")) || (0 == strcmp(pArg, "/d")))
			{
				iTimeUnit = DAY;
				DetermineOffsetTime(iTimeUnit, argv[ctr+1], lBegSrchTime, lEndSrchTime);
				ctr++;
			}
			else if((0 == strcmp(pArg, "-a")) || (0 == strcmp(pArg, "/a")))
			{
				bActualTime = true;
				FILETIME ftBeg, ftEnd;
				DetermineActualTime(iTimeUnit, argv[ctr+1], ftBeg, ftEnd);

				ftBegSrchActual = ftBeg;
				ftEndSrchActual = ftEnd;
				ctr++;
			}

			else if((0 == strcmp(pArg, "-?")) || (0 == strcmp(pArg, "/?")))
			{
				PrintHelp();
				return;
			}
			else
			{
				LPCTSTR pNewPath = (LPCTSTR)argv[ctr];
				BOOL bRetVal = ::SetCurrentDirectory(pNewPath);
			}
		}
	}
	
	//Subtract the user input
	tBegSearchTime = tCurrTime - lBegSrchTime;
	//Convert the time to a filetime

	TimetToFileTime(tBegSearchTime, &ftBegAccessSrch);

	if(!FileTimeToSystemTime(&ftBegAccessSrch, &stBegLFAccess))
		return;

		//test to see if we are searching an offset or a frame
		if(lEndSrchTime > lBegSrchTime)
		{
			g_bTimeFrame = false;
			//get the end times
			tEndSearchTime = tCurrTime - lEndSrchTime;
			//Convert the time to a filetime
			TimetToFileTime(tEndSearchTime, &ftEndAccessSrch);

			if(!FileTimeToSystemTime(&ftEndAccessSrch, &stEndLFAccess))
				return;
		}
	
	//print a start notice
	printf("Searching...\n");

	//Start the recursive search
	if(bActualTime)
		ScanDir(bScanSubs, pFileName, ftBegSrchActual, ftEndSrchActual);
	else
		ScanDir(bScanSubs, pFileName, ftBegAccessSrch, ftEndAccessSrch);

	//reset to original starting point
	::SetCurrentDirectory(CurDir);

	//print a closing notice
	char* pEndNotice = "\nFinished";
	printf("\r%-100s", pEndNotice);

	return;
}

	catch(...)
	{
		printf("\nError - Shutting Down...");
		return;
	}
}

void ScanDir(bool bScanSubs, char* pFileName, FILETIME ftBegSrchTime, FILETIME ftEndSrchTime)
{
	bool bPrintOnce = true;
	char* pFileSearch;

	//set search criteria
	pFileSearch = "*.*";

	//print the current dir so we can see where we are at
	char PrintDir[MAXPATH];
	::GetCurrentDirectory(sizeof(PrintDir), PrintDir);
	//zero out the last char to set a limit on the output
	PrintDir[79] = '\0';
	printf("\r%-79s", PrintDir);

	WIN32_FIND_DATA data;
	HANDLE hFile = ::FindFirstFile(pFileSearch, &data);
	//while(FindNextFile(hFile, &data))
	do
	{
		if(data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY
			&& data.cFileName[0] != '.' && data.cFileName[0] != '..' && bScanSubs)
		{
			::SetCurrentDirectory(data.cFileName);
			bPrintOnce = true;

			ScanDir(bScanSubs, pFileName, ftBegSrchTime, ftEndSrchTime);
			::SetCurrentDirectory("..");
		}
	
		else
		{
			FILETIME ftFileAccess;
			SYSTEMTIME stFileAccess;

			char* pTest = data.cFileName;
			if((0 == strcmp(pTest, ".."))||(0 == strcmp(pTest, ".")))
				continue;

			//Get the last-access time to local time.
			if(!FileTimeToLocalFileTime(&data.ftLastAccessTime, &ftFileAccess)) 
				return;

			//Convert the local file time from UTC to system time.
			if(!FileTimeToSystemTime(&ftFileAccess, &stFileAccess))
				return;

			//compare file access times 
			//if file access time is newer, print it
			int iRetVal = 0;

			if(g_bTimeFrame)
			{
				iRetVal = CompareFileTime(&ftFileAccess, &ftBegSrchTime);
				if((1 == iRetVal) || (0 == iRetVal))
				{
                    iRetVal = CompareFileTime(&ftFileAccess, &ftEndSrchTime);  					
						if((iRetVal == -1) || (iRetVal == 0))
						{
							//File has been accessed within our search time so print it
							//Build a string showing the date and time.
							//Group files by directory
							if(bPrintOnce)
							{
								char CurDir[MAXPATH];
								::GetCurrentDirectory(sizeof(CurDir), CurDir);
								printf("\r%s\n", CurDir);
								bPrintOnce = false;
							}
								TCHAR lpszString[1024];
								wsprintf(lpszString, "\t%s\t%02d/%02d/%d  %02d:%02d:%02d\n",
									data.cFileName,
									stFileAccess.wDay, stFileAccess.wMonth, stFileAccess.wYear,
									stFileAccess.wHour, stFileAccess.wMinute, stFileAccess.wSecond);
								printf(lpszString);

							

						}
				}
			}
			else
			{
				iRetVal = CompareFileTime(&data.ftLastAccessTime, &ftBegSrchTime);
				if(iRetVal == 1) //1 means second file is newer than first - HIGHER NUMBER == NEWER FILE
				{
					if(bPrintOnce)
					{
						char CurDir[MAXPATH];
						::GetCurrentDirectory(sizeof(CurDir), CurDir);
						printf("\r%s\n", CurDir);
						bPrintOnce = false;
					}

					//File has been accessed within our search time so print it
					//Build a string showing the date and time.
						TCHAR lpszString[1024];
						wsprintf(lpszString, "\t%s\t%02d/%02d/%d  %02d:%02d:%02d\n",
							data.cFileName,
							stFileAccess.wDay, stFileAccess.wMonth, stFileAccess.wYear,
							stFileAccess.wHour, stFileAccess.wMinute, stFileAccess.wSecond);
						printf(lpszString);
					
				}
			}
		}
	}while(FindNextFile(hFile, &data));

	//Close the handle
	::FindClose(hFile);
	
	return;
}

void DetermineOffsetTime(int iTimeUnit, char* pTimeFrame, long& lBegTimeTotal, long& lEndTimeTotal)
{
	int iBegAmount = 0;
	int iEndAmount = 0;
	//Determine if char param is dual times indicating a time frame
	char seps[]   = "-";
	char* pToken = NULL;
	char* pTimeFrameBeg = NULL;
	char* pTimeFrameEnd = NULL;

	// Establish string and get the first token
	pToken = strtok(pTimeFrame, seps); 
	pTimeFrameBeg = pToken;

	//Get next token 
	pToken = strtok(NULL, seps); 
	pTimeFrameEnd = pToken;

	//set the char string to integer
	//determine how many seconds - seconds * constant
	//return the time in seconds
	if(pTimeFrameBeg != NULL)
		iBegAmount = atoi(pTimeFrameBeg);
	else 
		iBegAmount = 0;

	if(pTimeFrameEnd!=NULL)
		iEndAmount = atoi(pTimeFrameEnd);
	else 
		iEndAmount = 0;

	if(iTimeUnit == SECOND) 
	{
		lBegTimeTotal = iBegAmount * SECOND;
		lEndTimeTotal = iEndAmount * SECOND;
	}
	if(iTimeUnit == MINUTE) 
	{
		lBegTimeTotal = iBegAmount * MINUTE;
		lEndTimeTotal = iEndAmount * MINUTE;
	}
	if(iTimeUnit == HOUR) 
	{
		lBegTimeTotal = iBegAmount * HOUR;
		lEndTimeTotal = iEndAmount * HOUR;
	}
	if(iTimeUnit == DAY) 
	{
		lBegTimeTotal = iBegAmount * DAY;
		lEndTimeTotal = iEndAmount * DAY;
	}

	return;
}

void DetermineActualTime(int iTimeUnit, char* pTimeFrame, FILETIME& ftBegTime, FILETIME& ftEndTime)
{
	int iBegAmount = 0;
	int iEndAmount = 0;
	//Determine if char param is dual times indicating a time frame
	char seps[]   = "/-:";
	char* pBegDay=NULL, *pBegMonth=NULL, *pBegYear=NULL, *pBegHour=NULL, *pBegMinute=NULL, *pBegSecond=NULL;
	char* pEndDay=NULL, *pEndMonth=NULL, *pEndyear=NULL, *pEndHour=NULL, *pEndMinute=NULL, *pEndSecond=NULL;
	char* pTimeFrameBeg = NULL;
	char* pTimeFrameEnd = NULL;
	SYSTEMTIME stBegSrchTime, stEndSrchTime;

	// Establish string and get the beginning date/time details
	pBegDay = strtok(pTimeFrame, seps); 
	pBegMonth = strtok(NULL, seps); 
	pBegYear = strtok(NULL, seps); 
	pBegHour = strtok(NULL, seps); 
	pBegMinute = strtok(NULL, seps); 
	pBegSecond = strtok(NULL, seps); 
	//get the end date/time details
	pEndDay = strtok(NULL, seps); 
	pEndMonth = strtok(NULL, seps); 
	pEndyear = strtok(NULL, seps); 
	pEndHour = strtok(NULL, seps); 
	pEndMinute = strtok(NULL, seps); 
	pEndSecond = strtok(NULL, seps); 
	//fill out beg system time struct
	try
	{
		//if a second date/time was passed
		//fill out ending system time struct
		if(pBegDay!=NULL)
		{
			stBegSrchTime.wDay = (WORD)atoi(pBegDay);   
			stBegSrchTime.wMonth = (WORD)atoi(pBegMonth); 
			stBegSrchTime.wYear = (WORD)atoi(pBegYear);  
			stBegSrchTime.wHour = (WORD)atoi(pBegHour);  
			stBegSrchTime.wMinute = (WORD)atoi(pBegMinute); 
			stBegSrchTime.wSecond = (WORD)atoi(pBegSecond);  
		}
		//if a second date/time was passed
		//fill out ending system time struct
		if(pEndDay != NULL)
		{
			stEndSrchTime.wDay = (WORD)atoi(pEndDay);   
			stEndSrchTime.wMonth = (WORD)atoi(pEndMonth);  
			stEndSrchTime.wYear = (WORD)atoi(pEndyear);
			stEndSrchTime.wHour = (WORD)atoi(pEndHour);  
			stEndSrchTime.wMinute = (WORD)atoi(pEndMinute); 
			stEndSrchTime.wSecond = (WORD)atoi(pEndSecond);  
		}
	}

	catch(...)
	{
		printf("Bad first date...use the form 'd/m/y-h/m/s' OR 'd/m/y-h/m/s-d/m/y-h:m:s'");
		exit(1);
	}
	//convert to file times and return the ref
	BOOL bRetVal = FALSE;
	bRetVal = SystemTimeToFileTime(&stBegSrchTime, &ftBegTime);
	if(!bRetVal)
	{
		printf("Bad first date...use the form 'd/m/y-h/m/s' OR 'd/m/y-h/m/s-d/m/y-h:m:s'");
		exit(1);
	}
	//test for second half of time frame
	if(pEndDay != NULL)
	{
		g_bTimeFrame = true;
		bRetVal = SystemTimeToFileTime(&stEndSrchTime, &ftEndTime);
		if(!bRetVal)
		{
			printf("Bad second date...use the form 'd/m/y-h/m/s' OR 'd/m/y-h/m/s-d/m/y-h:m:s'");
			exit(1);
		}
	}

	return;
}

void TimetToFileTime(time_t tTime, LPFILETIME pFileTime)
{
    LONGLONG llVal = Int32x32To64(tTime, 10000000) + 116444736000000000;
    pFileTime->dwLowDateTime = (DWORD)llVal;
    pFileTime->dwHighDateTime = (DWORD)(llVal >> 32);
}

void PrintHelp()
{
	cout << "AFind v1.4.2 - Copyright(c) 1998, NT OBJECTives, Inc." << endl;
	cout << "NTFS Last Access Time Finder" << endl;
	cout << "Programming by JD Glaser - All Rights Reserved" << endl;
	cout << "Command Line Switches" << endl;
	cout << "\t" << "[dirname]" << "\t" << "Directory to search" << endl;
	cout << "\t" << "-f [filename]" << "\t" << "List last access time of file" << endl;
	cout << "\t" << "-s [seconds]" << "\t" << "Files accessed less than x seconds ago" << endl;
	cout << "\t" << "-m [minutes]" << "\t" << "Files accessed less than x minutes ago" << endl;
	cout << "\t" << "-h [hours]" << "\t" << "Files accessed less than x hours ago" << endl;
	cout << "\t" << "-d [days]" << "\t" << "Files accessed less than x days ago" << endl;
	cout << "\t" << "-a [d/m/y-h:m:s]" << "\t" << "Files accessed after this date/time" << endl;
	cout << "\t" << "-ns" << "\t" << "Exclude sub-directories" << endl;
	cout << "\t" << "- or /" << "\t" << "Either switch statement can be used" << endl;
	cout << "\t" << "-?" << "\t" << "Help" << endl;
	cout << "\t" << "Additional time frame usage:" << endl;
	cout << "\t" << "afind /s 2-4  " << "Files accessed between 2 and 4 seconds ago " <<endl;
	cout << "\t" << "afind /m 2-4  " << "Files between 2 and 4 minutes ago " <<endl;
	cout << "\t" << "afind /s 2-4  " << "Files between 2 and 4 seconds ago " <<endl;
	cout << "\t" << "afind /a 14/7/1998-3:12:06-15/7/1998-2:05:30  " << "Files between these dates" <<endl;
	cout << "COMMAND PROMPT MUST HAVE A MINIMUM WIDTH OF 80 CHARACTERS" << endl;
	cout << "See http://www.ntobjectives.com for updates/fixes" << endl;
	return;
}
