//DACLchk v1.0 - Copyright(c) 1999 by JD Glaser - NT OBJECTives, Inc. ALL RIGHTS RESERVED
//NTFS DACL ACE Order Detector

 ********************************************************************
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 1, or (at your option)
 *  any later version. *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details. *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 



#include <tchar.h>
#include <stdio.h>
#include "filestat.h"

const int MAXPATH = 500;

void ScanDir(bool bScanSubs, CFileStat& fs);
void PrintHelp();

void main(int argc, char* argv[])
{
	bool bScanSubs = true;

	char CurDir[MAXPATH];

	// check if running on Windows NT, if not, display notice and terminate 
    if( GetVersion() & 0x80000000 ) 
    { 
      printf("DACLchk only runs on Windows NT...");
      return; 
    }

try
{
	//set thread priority to lowest
	HANDLE hThread;
	hThread = ::GetCurrentThread();
	::SetThreadPriority(hThread, THREAD_PRIORITY_LOWEST);

	if(argc < 2) // no args, I'm outa here
	{
		PrintHelp();
		return;
	}


	//Create our stat ob
	CFileStat fs;

	if(argc > 1)
	{
		for (int ctr = 1; ctr < argc; ctr++)
		{
			const char* pArg = argv[ctr];

			if((0 == strcmp(pArg, "-ns")) || (0 == strcmp(pArg, "/ns")))
			{
				bScanSubs = false;
			}

			if((0 == strcmp(pArg, "-d")) || (0 == strcmp(pArg, "/d")))
			{
				fs.SetDump();
			}

			else if((0 == strcmp(pArg, "-?")) || (0 == strcmp(pArg, "/?")))
			{
				PrintHelp();
				return;
			}

			else
			{
				char* pNewPath = argv[ctr];
				::SetCurrentDirectory(pNewPath);
			}
		}
	}
	else
	{
		::GetCurrentDirectory(sizeof(CurDir), CurDir);
	}	

	printf("Detecting...\n");
	//Start the recursive search
	ScanDir(bScanSubs, fs);

	//reset to original starting point
	::SetCurrentDirectory(CurDir);

	//print a closing notice
	char* pEndNotice = "Finished Detecting...";
	printf("\r%-79s", pEndNotice);

	return;
}

	catch(...)
	{
		printf("\nError - Shutting Down...");
		return;
	}
}

void ScanDir(bool bScanSubs, CFileStat& fs)
{
	bool bPrintOnce = true;

	WIN32_FIND_DATA data;
	HANDLE hFile = ::FindFirstFile("*.*", &data);
	while(FindNextFile(hFile, &data))
	{
		if(data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY
			&& data.cFileName[0] != '.' && bScanSubs)
		{
			::SetCurrentDirectory(data.cFileName);
			bPrintOnce = true;

			//scan into the next sub-directory
			ScanDir(bScanSubs, fs);
			::SetCurrentDirectory("..");
		}
	
		else
		{
			FILETIME ftLocal;
			SYSTEMTIME stCreate;

			//Get the last-access time to local time.
			if (!FileTimeToLocalFileTime(&data.ftLastAccessTime, &ftLocal)) 
				return;
			//Convert the local file time from UTC to system time.
			FileTimeToSystemTime(&ftLocal, &stCreate);

			//Group files by directory
			if(bPrintOnce)
			{
				char CurDir[MAXPATH];
				::GetCurrentDirectory(sizeof(CurDir), CurDir);

				//zero out the last char to set a limit on the output
				CurDir[79] = '\0';
				printf("\r%-79s", CurDir);
				bPrintOnce = false;
			}
			//Grab file info here
			fs.RecordSecStats(data.cFileName);
			fs.PrintStats();
			fs.Reset();
			
		}
	}
	//Close the handle
	::FindClose(hFile);
};

void PrintHelp()
{
	cout << "DACLchk v1.2 - Copyright(c) 1999, NT OBJECTives, Inc." << endl;
	cout << "NTFS DACL ACE Order Detector" << endl;
	cout << "Dumps any ACL that has Denied and Allowed ACE's in reverse order" << endl;
	cout << "Programming by JD Glaser - All Rights Reserved" << endl;
	cout << "\t" << "Usage - sfind [path] /ns" << endl;
	cout << "\t" << "[dirpath]" << "\t" << "Directory to search - none equals current" << endl;
	cout << "\t" << "-d" << "\t\t" << "Dump all DACL's - Don't detect reversed ACE's" << endl;
	cout << "\t" << "-ns" << "\t\t" << "Skip sub-directories" << endl;
	cout << "\t" << "- or /" << "\t\t" << "Either switch statement can be used" << endl;
	cout << "\t" << "-?" << "\t\t" << "Help" << endl;
	cout << "COMMAND PROMPT MUST HAVE A MINIMUM WIDTH OF 80 CHARACTERS" << endl;
	cout << "See http://www.ntobjectives.com for updates/fixes" << endl;
	return;
}