//FileStat v1.0 - Copyright 1998 by JD Glaser - NT OBJECTives, Inc. ALL RIGHTS RESERVED
//CFileStat.cpp

// plus MSDN structs and init code

 ********************************************************************
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 1, or (at your option)
 *  any later version. *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details. *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 



#include "FileStat.h"

IMPLEMENT_SERIAL(CFileStat, CObject, 1)

CFileStat::CFileStat()
{
	m_iAllowedCount = 0;
	m_iDeniedCount = 0;
	m_iFileCount = 0;
	m_iBADdaclCount = 0;
	m_bDisordered = false;
	m_bDumpDACL = false;

	//give privs to scan files to which you have no permissions
	EnableBCKprivs();

	//set us up 
	InitializeWellKnownSIDs(); 

	//Give us access to SACL
	SetPrivilegeInAccessToken(); 
}

CFileStat::~CFileStat()
{

}

BOOL CFileStat::SetPrivilegeInAccessToken(VOID) 
{ 
  //This process is needed to obtain privilege 
  //to examine the SACL audit info

  HANDLE           hProcess; 
  HANDLE           hAccessToken; 
  LUID             luidPrivilegeLUID; 
  TOKEN_PRIVILEGES tpTokenPrivilege; 
 
  hProcess = GetCurrentProcess(); 
  if(!hProcess)
	{return FALSE;}

  if (!OpenProcessToken(hProcess, 
                    TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, 
                    &hAccessToken)) 
	{return FALSE;}
 
 
  if (!LookupPrivilegeValue(NULL, SE_SECURITY_NAME, 
                            &luidPrivilegeLUID)) 
	{return FALSE;}

 
  tpTokenPrivilege.PrivilegeCount = 1; 
  tpTokenPrivilege.Privileges[0].Luid = luidPrivilegeLUID; 
  tpTokenPrivilege.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED; 
 
  AdjustTokenPrivileges (hAccessToken, 
                         FALSE,   
                         &tpTokenPrivilege, 
                         sizeof(TOKEN_PRIVILEGES), 
                         NULL,  
                         NULL); 
  if(GetLastError()!= NO_ERROR)
	{return FALSE;}
 
  return TRUE; 
} 

VOID CFileStat::InitializeWellKnownSIDs(VOID) 
{ 
  DWORD dwSidWith0SubAuthorities; 
  DWORD dwSidWith1SubAuthority; 
  DWORD dwSidWith2SubAuthorities; 
  DWORD dwSidWith3SubAuthorities; 
  DWORD dwSidWith4SubAuthorities; 
 
  SID_IDENTIFIER_AUTHORITY siaNullSidAuthority    = SECURITY_NULL_SID_AUTHORITY; 
  SID_IDENTIFIER_AUTHORITY siaWorldSidAuthority   = SECURITY_WORLD_SID_AUTHORITY; 
  SID_IDENTIFIER_AUTHORITY siaLocalSidAuthority   = SECURITY_LOCAL_SID_AUTHORITY; 
  SID_IDENTIFIER_AUTHORITY siaCreatorSidAuthority = SECURITY_CREATOR_SID_AUTHORITY; 
  SID_IDENTIFIER_AUTHORITY siaNtAuthority         = SECURITY_NT_AUTHORITY; 
 
  //  These SID sizes need to be allocated 
  dwSidWith0SubAuthorities = GetSidLengthRequired( 0 ); 
  dwSidWith1SubAuthority   = GetSidLengthRequired( 1 ); 
  dwSidWith2SubAuthorities = GetSidLengthRequired( 2 ); 
  dwSidWith3SubAuthorities = GetSidLengthRequired( 3 ); 
  dwSidWith4SubAuthorities = GetSidLengthRequired( 4 ); 
 
  //  Allocate and initialize the universal SIDs 
  m_psidNullSid         = (PSID)LocalAlloc(LPTR,dwSidWith1SubAuthority); 
  m_psidWorldSid        = (PSID)LocalAlloc(LPTR,dwSidWith1SubAuthority); 
  m_psidLocalSid        = (PSID)LocalAlloc(LPTR,dwSidWith1SubAuthority); 
  m_psidCreatorOwnerSid = (PSID)LocalAlloc(LPTR,dwSidWith1SubAuthority); 
  m_psidCreatorGroupSid = (PSID)LocalAlloc(LPTR,dwSidWith1SubAuthority); 
 
  InitializeSid( m_psidNullSid,         &siaNullSidAuthority,    1 ); 
  InitializeSid( m_psidWorldSid,        &siaWorldSidAuthority,   1 ); 
  InitializeSid( m_psidLocalSid,        &siaLocalSidAuthority,   1 ); 
  InitializeSid( m_psidCreatorOwnerSid, &siaCreatorSidAuthority, 1 ); 
  InitializeSid( m_psidCreatorGroupSid, &siaCreatorSidAuthority, 1 ); 
 
  *(GetSidSubAuthority( m_psidNullSid,         0 )) = SECURITY_NULL_RID; 
  *(GetSidSubAuthority( m_psidWorldSid,        0 )) = SECURITY_WORLD_RID; 
  *(GetSidSubAuthority( m_psidLocalSid,        0 )) = SECURITY_LOCAL_RID; 
  *(GetSidSubAuthority( m_psidCreatorOwnerSid, 0 )) = SECURITY_CREATOR_OWNER_RID; 
  *(GetSidSubAuthority( m_psidCreatorGroupSid, 0 )) = SECURITY_CREATOR_GROUP_RID; 
 
  // Allocate and initialize the NT defined SIDs 
  m_psidNtAuthoritySid   = (PSID)LocalAlloc(LPTR,dwSidWith0SubAuthorities); 
  m_psidDialupSid        = (PSID)LocalAlloc(LPTR,dwSidWith1SubAuthority); 
  m_psidNetworkSid       = (PSID)LocalAlloc(LPTR,dwSidWith1SubAuthority); 
  m_psidBatchSid         = (PSID)LocalAlloc(LPTR,dwSidWith1SubAuthority); 
  m_psidInteractiveSid   = (PSID)LocalAlloc(LPTR,dwSidWith1SubAuthority); 
  m_psidLogonIdsSid      = (PSID)LocalAlloc(LPTR,dwSidWith3SubAuthorities); 
  m_psidServiceSid       = (PSID)LocalAlloc(LPTR,dwSidWith1SubAuthority); 
  m_psidLocalSystemSid   = (PSID)LocalAlloc(LPTR,dwSidWith1SubAuthority); 
  m_psidBuiltinDomainSid = (PSID)LocalAlloc(LPTR,dwSidWith1SubAuthority); 
 
  InitializeSid( m_psidNtAuthoritySid,   &siaNtAuthority, 0 ); 
  InitializeSid( m_psidDialupSid,        &siaNtAuthority, 1 ); 
  InitializeSid( m_psidNetworkSid,       &siaNtAuthority, 1 ); 
  InitializeSid( m_psidBatchSid,         &siaNtAuthority, 1 ); 
  InitializeSid( m_psidInteractiveSid,   &siaNtAuthority, 1 ); 
  InitializeSid( m_psidLogonIdsSid,      &siaNtAuthority, 3 ); 
  InitializeSid( m_psidServiceSid,       &siaNtAuthority, 1 ); 
  InitializeSid( m_psidLocalSystemSid,   &siaNtAuthority, 1 ); 
  InitializeSid( m_psidBuiltinDomainSid, &siaNtAuthority, 1 ); 
 
  *(GetSidSubAuthority( m_psidDialupSid,        0 )) = SECURITY_DIALUP_RID; 
  *(GetSidSubAuthority( m_psidNetworkSid,       0 )) = SECURITY_NETWORK_RID; 
  *(GetSidSubAuthority( m_psidBatchSid,         0 )) = SECURITY_BATCH_RID; 
  *(GetSidSubAuthority( m_psidInteractiveSid,   0 )) = SECURITY_INTERACTIVE_RID; 
  *(GetSidSubAuthority( m_psidLogonIdsSid,      0 )) = SECURITY_LOGON_IDS_RID; 
  *(GetSidSubAuthority( m_psidLogonIdsSid,      1 )) = 0; // Bogus! 
  *(GetSidSubAuthority( m_psidLogonIdsSid,      2 )) = 0; // Also bogus! 
  *(GetSidSubAuthority( m_psidServiceSid,       0 )) = SECURITY_SERVICE_RID; 
  *(GetSidSubAuthority( m_psidLocalSystemSid,   0 )) = SECURITY_LOCAL_SYSTEM_RID; 
  *(GetSidSubAuthority( m_psidBuiltinDomainSid, 0 )) = SECURITY_BUILTIN_DOMAIN_RID; 
} 

BOOL CFileStat::SIDStringName(PSID psidSID, LPTSTR lpszSIDStringName) 
{ 
 
  DWORD dwNumSubAuthorities; 
  DWORD dwLen; 
  DWORD dwSubAuthorityI; 
  #define BytesInIdentifierAuthority  6 
  SID_IDENTIFIER_AUTHORITY siaSidAuthority; 
  SID_IDENTIFIER_AUTHORITY siaNullSidAuthority    = SECURITY_NULL_SID_AUTHORITY; 
  SID_IDENTIFIER_AUTHORITY siaWorldSidAuthority   = SECURITY_WORLD_SID_AUTHORITY; 
  SID_IDENTIFIER_AUTHORITY siaLocalSidAuthority   = SECURITY_LOCAL_SID_AUTHORITY; 
  SID_IDENTIFIER_AUTHORITY siaCreatorSidAuthority = SECURITY_CREATOR_SID_AUTHORITY; 
  SID_IDENTIFIER_AUTHORITY siaNtAuthority         = SECURITY_NT_AUTHORITY; 
 
  dwLen = sprintf(lpszSIDStringName,"S-%d-",SID_REVISION); 
 
  if (SID_REVISION != ((PISID)psidSID)->Revision) 
  { dwLen += sprintf(lpszSIDStringName+dwLen,"bad_revision==%d",((PISID)psidSID)->Revision); 
  } 
 
  siaSidAuthority = *(GetSidIdentifierAuthority(psidSID)); 
 
  if      (0==memcmp(&siaSidAuthority,&siaNullSidAuthority   ,BytesInIdentifierAuthority)) 
  { dwLen += sprintf(lpszSIDStringName+dwLen,"0"); 
  } 
  else if (0==memcmp(&siaSidAuthority,&siaWorldSidAuthority  ,BytesInIdentifierAuthority)) 
  { dwLen += sprintf(lpszSIDStringName+dwLen,"1"); 
  } 
  else if (0==memcmp(&siaSidAuthority,&siaLocalSidAuthority  ,BytesInIdentifierAuthority)) 
  { dwLen += sprintf(lpszSIDStringName+dwLen,"2"); 
  } 
  else if (0==memcmp(&siaSidAuthority,&siaCreatorSidAuthority,BytesInIdentifierAuthority)) 
  { dwLen += sprintf(lpszSIDStringName+dwLen,"3"); 
  } 
  else if (0==memcmp(&siaSidAuthority,&siaNtAuthority        ,BytesInIdentifierAuthority)) 
  { dwLen += sprintf(lpszSIDStringName+dwLen,"5"); 
  } 
  else 
  { dwLen += sprintf(lpszSIDStringName+dwLen,"UnknownAuthority!"); 
  } 
 
  dwNumSubAuthorities = (DWORD)( *(GetSidSubAuthorityCount(psidSID)) ); 
 
  for (dwSubAuthorityI=0; dwSubAuthorityI<dwNumSubAuthorities; dwSubAuthorityI++) 
  { dwLen += sprintf(lpszSIDStringName+dwLen,"-%d",*(GetSidSubAuthority(psidSID,dwSubAuthorityI))); 
  } 
 
  return(TRUE); 
}

void CFileStat::EnableBCKprivs()
{	
	HANDLE hToken;
	BYTE byteBuff[sizeof TOKEN_PRIVILEGES* 2];
	TOKEN_PRIVILEGES& tkp = *((TOKEN_PRIVILEGES*)byteBuff);

	OpenProcessToken( GetCurrentProcess(),
		TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hToken); 
	// set up SeBackupPrivilege, SeRestorePrivilege
	LookupPrivilegeValue(NULL, SE_BACKUP_NAME, &tkp.Privileges[0].Luid);

	LookupPrivilegeValue(NULL, SE_RESTORE_NAME, &tkp.Privileges[1].Luid);

	tkp.PrivilegeCount = 2;
	tkp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
	tkp.Privileges[1].Attributes = SE_PRIVILEGE_ENABLED;
	AdjustTokenPrivileges(hToken, FALSE, &tkp, sizeof tkp, NULL, NULL);
}

int CFileStat::PrintStats()
{
	if(m_strFileName == "..")
		return 0;
	
	if(m_bDumpDACL || m_bDisordered)
	{
		//print the first entry always
		//if dumping, print the rest as well
		POSITION pos;
		pos = m_PrintList.GetHeadPosition();
		printf("  ");
		printf("\n  %s", m_PrintList.GetNext(pos)); // print the file name
		printf(" %s", m_PrintList.GetNext(pos)); //print the number of ACE's

			while(pos != NULL) 
			{
			 printf("%s", m_PrintList.GetNext(pos)); // print the ACE's
			}
		printf("\n\t\tACL done...\n\n"); // cap off the end of list
	}

	return true;
}

int CFileStat::RecordSecStats(LPTSTR lpszFullName)
{
	#define              SZ_SD_BUF 8096 
	UCHAR                ucBuf[SZ_SD_BUF]; 
	DWORD                dwSDLength = SZ_SD_BUF; 
	DWORD                dwSDLengthNeeded = 0; 
	PSECURITY_DESCRIPTOR psdSD      = (PSECURITY_DESCRIPTOR)&ucBuf; 
	DWORD                dwError = 0; 
	int					 retVal = 0;


	m_strFileName = lpszFullName;

	retVal = GetFileSecurity(lpszFullName, 
							(SECURITY_INFORMATION)( OWNER_SECURITY_INFORMATION 
							   | GROUP_SECURITY_INFORMATION 
							   | DACL_SECURITY_INFORMATION 
							   | SACL_SECURITY_INFORMATION), 
					psdSD, 
					dwSDLength, 
					(LPDWORD)&dwSDLengthNeeded);
	 
	dwError = GetLastError();

	if(!ExamineSD(psdSD, FileAccessMask));
	{return FALSE;}

	dwError = GetLastError();

	return true;

}

BOOL CFileStat::ExamineSD(PSECURITY_DESCRIPTOR psdSD, 
                          KINDS_OF_ACCESSMASKS_DECODED kamKindOfMask) 
{ 
 
  PACL                        paclDACL; 
  BOOL                        bHasDACL        = FALSE; 
  BOOL                        bDaclDefaulted  = FALSE; 
  SECURITY_DESCRIPTOR_CONTROL sdcSDControl; 
  DWORD                       dwSDRevision; 
  DWORD                       dwSDLength; 
 
  if (!IsValidSecurityDescriptor(psdSD)) 
  {return FALSE;} 
 
  dwSDLength = GetSecurityDescriptorLength(psdSD); 
 
  if (!GetSecurityDescriptorDacl(psdSD, 
                                 (LPBOOL)&bHasDACL, 
                                 (PACL *)&paclDACL, 
                                 (LPBOOL)&bDaclDefaulted)) 
  {return FALSE;} 
 
   if (!GetSecurityDescriptorControl(psdSD, 
                                    (PSECURITY_DESCRIPTOR_CONTROL)&sdcSDControl, 
                                    (LPDWORD)&dwSDRevision)) 
  {return FALSE;} 
 
    if(NULL == paclDACL) 
    {
		//Handle NULL errors here
		printf("\n%s SD has a NULL DACL which explicitly allows all access to Everyone", m_strFileName); 
    } 


	//HERE IS WERE WE DO THE EXAM WORK
    else if(!ExamineACL(paclDACL,"",kamKindOfMask)) 
    { 
		//handle the error here
		printf("File - %s - has a corrupt ACL", m_strFileName);
		return FALSE;
	} 

  return TRUE;
} 

BOOL CFileStat::ExamineACL(PACL paclACL, LPTSTR lpszOldIndent, 
							KINDS_OF_ACCESSMASKS_DECODED kamKindOfMask) 
{ 
  #define						SZ_INDENT_BUF 80 
  UCHAR							ucIndentBuf[SZ_INDENT_BUF] = ""; 
  ACL_SIZE_INFORMATION          asiAclSize; 
  ACL_REVISION_INFORMATION      ariAclRevision; 
  DWORD							dwBufLength; 
  DWORD							dwAcl_i; 
  ACCESS_ALLOWED_ACE*			paaAllowedAce; 

  CString aclLine;
 
  strcpy((char*)ucIndentBuf, lpszOldIndent); 
  strcat((char*)ucIndentBuf,"    "); 
 
  if (!IsValidAcl(paclACL)) 
  {  
    return(FALSE); 
  } 
 
  dwBufLength = sizeof(asiAclSize); 
 
  if (!GetAclInformation(paclACL, 
                         (LPVOID)&asiAclSize, 
                         (DWORD)dwBufLength, 
                         (ACL_INFORMATION_CLASS)AclSizeInformation)) 
  { 
    return(FALSE); 
  } 
 
	//PRINT the file name
	m_PrintList.AddTail(m_strFileName);

	//PRINT the count of ace
	aclLine.Format("  ACL has %d ACE(s)", asiAclSize.AceCount);
	m_PrintList.AddTail(aclLine);

  
  dwBufLength = sizeof(ariAclRevision); 
 
  if (!GetAclInformation(paclACL, 
                         (LPVOID)&ariAclRevision, 
                         (DWORD)dwBufLength, 
                         (ACL_INFORMATION_CLASS)AclRevisionInformation)) 
  {  
    return(FALSE); 
  } 
 
  for (dwAcl_i = 0; dwAcl_i < asiAclSize.AceCount;  dwAcl_i++) 
  { 
    if (!GetAce(paclACL, 
                dwAcl_i, 
                (LPVOID *)&paaAllowedAce)) 
    {  
      return FALSE; 
    } 

    if(!LookupSIDName((PSID)&(paaAllowedAce->SidStart),(char*)ucIndentBuf)) 
    { return FALSE; } 
 
	DWORD dwAceFlags = paaAllowedAce->Header.AceFlags; 

    switch (paaAllowedAce->Header.AceType) 
    { 
      case                       ACCESS_ALLOWED_ACE_TYPE : 
      { 
		aclLine.Format("\n%s  ACE %d is an ACCESS_ALLOWED_ACE_TYPE",ucIndentBuf,dwAcl_i);
		m_PrintList.AddTail(aclLine);
		m_iAllowedCount++;
        break; 
      } 
      case                       ACCESS_DENIED_ACE_TYPE : 
      {
		aclLine.Format("\n%s  ACE %d is an ACCESS_DENIED_ACE_TYPE",ucIndentBuf,dwAcl_i); 
		m_PrintList.AddTail(aclLine);
		m_iDeniedCount++;
		if(m_iAllowedCount)
		{
			  printf("\nThis ACL is disordered");
			  m_bDisordered = true;
		}
        break; 
      } 
      default : 
      { 
		aclLine.Format("\n%s  ACE %d is BOGUS!!! Better Check this out!",ucIndentBuf,dwAcl_i); 
		m_PrintList.AddTail(aclLine);
		return FALSE; 
      } 
    } 


    aclLine.Format("\n%s  ACE %d mask = 0x%.8x",ucIndentBuf,dwAcl_i,paaAllowedAce->Mask); 
	m_PrintList.AddTail(aclLine);

	//break up the mask and show what perms are really there
	ExamineMask(paaAllowedAce->Mask,kamKindOfMask);

  } 

  return TRUE;
} 

BOOL CFileStat::LookupSIDName(PSID psidSID, LPTSTR lpszOldIndent) 
{ 
  UCHAR        ucIndentBuf    [SZ_INDENT_BUF]    = ""; 
  #define                      SZ_ACCT_NAME_BUF  60 
  UCHAR        ucNameBuf      [SZ_ACCT_NAME_BUF] = ""; 
  DWORD        dwNameLength  = SZ_ACCT_NAME_BUF; 
  #define                      SZ_DMN_NAME_BUF   60 
  UCHAR        ucDomainNmBuf  [SZ_DMN_NAME_BUF]  = ""; 
  DWORD        dwDNameLength = SZ_DMN_NAME_BUF; 
  #define                      SZ_SID_STRING_BUF 150 
  UCHAR        ucSIDStringBuf [SZ_SID_STRING_BUF] = ""; 
  SID_NAME_USE peAcctNameUse = SidTypeInvalid; 
  DWORD        dwLookupStatus; 
  BOOL         bGotBadLookupThatIsNotLocalLogonSID; 
 
	CString nameLine;

	strcpy((char*)ucIndentBuf,lpszOldIndent); 
	strcat((char*)ucIndentBuf,"  "); 
 
  if (!IsValidSid(psidSID)) 
  { 
    return(FALSE); 
  } 
 
  if (!SIDStringName(psidSID, (char*)ucSIDStringBuf)) 
  {
    return FALSE; 
  } 
 
  if (!LookupAccountSid( 
         (LPTSTR)"",         // Look on local machine 
         psidSID, 
         (LPTSTR)&ucNameBuf, 
         (LPDWORD)&dwNameLength, 
         (LPTSTR)&ucDomainNmBuf, 
         (LPDWORD)&dwDNameLength, 
         (PSID_NAME_USE)&peAcctNameUse)) 
  { 
    dwLookupStatus = GetLastError(); 
 
 
    // Must have same number of sub authorities 
     bGotBadLookupThatIsNotLocalLogonSID = FALSE;  
 
    if ( ( *(GetSidSubAuthorityCount(m_psidLogonIdsSid))) != 
         ( *(GetSidSubAuthorityCount(psidSID        )))    ) 
    { // Not same number of sub-authorities, so can't be a match 
      bGotBadLookupThatIsNotLocalLogonSID = TRUE; 
    } 
    else 
    { 
      // Force the last two sub-authorities to match 
      *(GetSidSubAuthority( m_psidLogonIdsSid, 1 )) = 
      *(GetSidSubAuthority( psidSID        , 1 )); 
      *(GetSidSubAuthority( m_psidLogonIdsSid, 2 )) = 
      *(GetSidSubAuthority( psidSID        , 2 )); 
 
 
      if (EqualSid(psidSID, m_psidLogonIdsSid)) 
      { 
		nameLine.Format("\n    %sSID is the Local Logon SID   %s",ucIndentBuf,ucSIDStringBuf); 
   		m_PrintList.AddTail(nameLine);

	  } 
      else 
      { bGotBadLookupThatIsNotLocalLogonSID = TRUE; 
      } 
	} 
    if (bGotBadLookupThatIsNotLocalLogonSID) 
    { 
      if (ERROR_NONE_MAPPED == dwLookupStatus) 
      { 
		  nameLine.Format("\n    SID = %s/%s    (Account Unknown)  %s",ucDomainNmBuf,ucNameBuf,ucSIDStringBuf); 
 		  m_PrintList.AddTail(nameLine);
	  } 
      else 
      {
        return FALSE; 
      } 
    } 
  } 
	else 
	{ // Got good Lookup, so SID Is NOT the Local Logon SID 
		nameLine.Format("\n    SID = %s/%s   %s",ucDomainNmBuf,ucNameBuf,ucSIDStringBuf); 
		m_PrintList.AddTail(nameLine);
	} 

  return TRUE;
} 

void CFileStat::ExamineMask(ACCESS_MASK amMask, KINDS_OF_ACCESSMASKS_DECODED kamKindOfMask)
{
  #define STANDARD_RIGHTS_ALL_THE_BITS 0x00FF0000L
  #define GENERIC_RIGHTS_ALL_THE_BITS  0xF0000000L
  DWORD dwGenericBits;
  DWORD dwStandardBits;
  DWORD dwSpecificBits;
  DWORD dwAccessSystemSecurityBit;

  dwStandardBits            = (amMask); //& STANDARD_RIGHTS_ALL_THE_BITS);
  dwSpecificBits            = (amMask); //& SPECIFIC_RIGHTS_ALL         );
  dwAccessSystemSecurityBit = (amMask); //& ACCESS_SYSTEM_SECURITY      );
  dwGenericBits             = (amMask); //& GENERIC_RIGHTS_ALL_THE_BITS );

  //Print then decode the specific rights bits
  CString rightsLine;

  if (dwSpecificBits)
  {
    if(FileAccessMask == kamKindOfMask)
    {

      if (((dwStandardBits | dwSpecificBits   )
           & FILE_GENERIC_READ   ) == FILE_GENERIC_READ   )
      { 
		 rightsLine.Format("%s", " -R");
			m_PrintList.AddTail(rightsLine);
      }
      if (((dwStandardBits | dwSpecificBits   )
           & FILE_GENERIC_WRITE  ) == FILE_GENERIC_WRITE  )
      { 
  		  rightsLine.Format("%s", " -W");
			m_PrintList.AddTail(rightsLine);
      }
      if (((dwStandardBits | dwSpecificBits   )
           & FILE_GENERIC_EXECUTE) == FILE_GENERIC_EXECUTE)
      { 
  		  rightsLine.Format("%s", " -X");
			m_PrintList.AddTail(rightsLine);
      }
		if ((dwStandardBits    & DELETE  ) == DELETE  )
		{ 
  			rightsLine.Format("%s", " -D");
			m_PrintList.AddTail(rightsLine);
		}
      if ((dwSpecificBits    & FILE_DELETE_CHILD ) == FILE_DELETE_CHILD  )
      { 
			rightsLine.Format("%s", " -DEL_CHILD");
			m_PrintList.AddTail(rightsLine);
      }
		if ((dwStandardBits    & WRITE_DAC  ) == WRITE_DAC )
		{ 
  			rightsLine.Format("%s", " -CHANGE_PERMS");
			m_PrintList.AddTail(rightsLine);
		}
		if ((dwStandardBits    & WRITE_OWNER ) == WRITE_OWNER )
		{ 
  			rightsLine.Format("%s", " -TAKE_OWN");
			m_PrintList.AddTail(rightsLine);
		}

    }
  }

}

void CFileStat::Reset()
{
	m_iAllowedCount = 0;
	m_iDeniedCount = 0;
	m_bDisordered = false;
	m_PrintList.RemoveAll();
}

void CFileStat::SetDump()
{
	m_bDumpDACL = true;
}
