//FileStat v1.0 - Copyright 1998 by JD Glaser - NT OBJECTives, Inc. ALL RIGHTS RESERVED
//CFileStat.cpp

//plus MSDN structs and init code and values

 ********************************************************************
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 1, or (at your option)
 *  any later version. *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details. *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 


#include <iostream.h>
#include "FileStat.h"

IMPLEMENT_SERIAL(CFileStat, CObject, 1)

CFileStat::CFileStat()
{
	m_dwFileAttributes = 0;
	m_lStreamCount = 0;
    m_dwFileSize = 0;

	m_bArchive = FALSE;	
	m_bCompressed = FALSE;	
	m_bDirectory = FALSE;	
	m_bHidden = FALSE;		
	m_bNormal = FALSE;		
	m_bOffLine = FALSE;	
	m_bReadOnly = FALSE;	
	m_bSystem = FALSE;		
	m_bTemp = FALSE;
	
	//give privs to scan files to which you have no permissions
	EnableBCKprivs();

	//set us up 
	InitializeWellKnownSIDs(); 

	//Give us access to SACL
	SetPrivilegeInAccessToken(); 
}

CFileStat::~CFileStat()
{
}

BOOL CFileStat::SetPrivilegeInAccessToken(VOID) 
{ 
  //This process is needed to obtain privilege 
  //to examine the SACL audit info

  HANDLE           hProcess; 
  HANDLE           hAccessToken; 
  LUID             luidPrivilegeLUID; 
  TOKEN_PRIVILEGES tpTokenPrivilege; 
 
  hProcess = GetCurrentProcess(); 
  if(!hProcess)
	{return FALSE;}

  if (!OpenProcessToken(hProcess, 
                    TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, 
                    &hAccessToken)) 
	{return FALSE;}
 
 
  if (!LookupPrivilegeValue(NULL, SE_SECURITY_NAME, 
                            &luidPrivilegeLUID)) 
	{return FALSE;}

 
  tpTokenPrivilege.PrivilegeCount = 1; 
  tpTokenPrivilege.Privileges[0].Luid = luidPrivilegeLUID; 
  tpTokenPrivilege.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED; 
 
  AdjustTokenPrivileges (hAccessToken, 
                         FALSE,   
                         &tpTokenPrivilege, 
                         sizeof(TOKEN_PRIVILEGES), 
                         NULL,  
                         NULL); 
  if(GetLastError()!= NO_ERROR)
	{return FALSE;}
 
  return TRUE; 
} 

VOID CFileStat::InitializeWellKnownSIDs(VOID) 
{ 
  DWORD dwSidWith0SubAuthorities; 
  DWORD dwSidWith1SubAuthority; 
  DWORD dwSidWith2SubAuthorities; 
  DWORD dwSidWith3SubAuthorities; 
  DWORD dwSidWith4SubAuthorities; 
 
  SID_IDENTIFIER_AUTHORITY siaNullSidAuthority    = SECURITY_NULL_SID_AUTHORITY; 
  SID_IDENTIFIER_AUTHORITY siaWorldSidAuthority   = SECURITY_WORLD_SID_AUTHORITY; 
  SID_IDENTIFIER_AUTHORITY siaLocalSidAuthority   = SECURITY_LOCAL_SID_AUTHORITY; 
  SID_IDENTIFIER_AUTHORITY siaCreatorSidAuthority = SECURITY_CREATOR_SID_AUTHORITY; 
  SID_IDENTIFIER_AUTHORITY siaNtAuthority         = SECURITY_NT_AUTHORITY; 
 
  //  These SID sizes need to be allocated 
  dwSidWith0SubAuthorities = GetSidLengthRequired( 0 ); 
  dwSidWith1SubAuthority   = GetSidLengthRequired( 1 ); 
  dwSidWith2SubAuthorities = GetSidLengthRequired( 2 ); 
  dwSidWith3SubAuthorities = GetSidLengthRequired( 3 ); 
  dwSidWith4SubAuthorities = GetSidLengthRequired( 4 ); 
 
  //  Allocate and initialize the universal SIDs 
  m_psidNullSid         = (PSID)LocalAlloc(LPTR,dwSidWith1SubAuthority); 
  m_psidWorldSid        = (PSID)LocalAlloc(LPTR,dwSidWith1SubAuthority); 
  m_psidLocalSid        = (PSID)LocalAlloc(LPTR,dwSidWith1SubAuthority); 
  m_psidCreatorOwnerSid = (PSID)LocalAlloc(LPTR,dwSidWith1SubAuthority); 
  m_psidCreatorGroupSid = (PSID)LocalAlloc(LPTR,dwSidWith1SubAuthority); 
 
  InitializeSid( m_psidNullSid,         &siaNullSidAuthority,    1 ); 
  InitializeSid( m_psidWorldSid,        &siaWorldSidAuthority,   1 ); 
  InitializeSid( m_psidLocalSid,        &siaLocalSidAuthority,   1 ); 
  InitializeSid( m_psidCreatorOwnerSid, &siaCreatorSidAuthority, 1 ); 
  InitializeSid( m_psidCreatorGroupSid, &siaCreatorSidAuthority, 1 ); 
 
  *(GetSidSubAuthority( m_psidNullSid,         0 )) = SECURITY_NULL_RID; 
  *(GetSidSubAuthority( m_psidWorldSid,        0 )) = SECURITY_WORLD_RID; 
  *(GetSidSubAuthority( m_psidLocalSid,        0 )) = SECURITY_LOCAL_RID; 
  *(GetSidSubAuthority( m_psidCreatorOwnerSid, 0 )) = SECURITY_CREATOR_OWNER_RID; 
  *(GetSidSubAuthority( m_psidCreatorGroupSid, 0 )) = SECURITY_CREATOR_GROUP_RID; 
 
  // Allocate and initialize the NT defined SIDs 
  m_psidNtAuthoritySid   = (PSID)LocalAlloc(LPTR,dwSidWith0SubAuthorities); 
  m_psidDialupSid        = (PSID)LocalAlloc(LPTR,dwSidWith1SubAuthority); 
  m_psidNetworkSid       = (PSID)LocalAlloc(LPTR,dwSidWith1SubAuthority); 
  m_psidBatchSid         = (PSID)LocalAlloc(LPTR,dwSidWith1SubAuthority); 
  m_psidInteractiveSid   = (PSID)LocalAlloc(LPTR,dwSidWith1SubAuthority); 
  m_psidLogonIdsSid      = (PSID)LocalAlloc(LPTR,dwSidWith3SubAuthorities); 
  m_psidServiceSid       = (PSID)LocalAlloc(LPTR,dwSidWith1SubAuthority); 
  m_psidLocalSystemSid   = (PSID)LocalAlloc(LPTR,dwSidWith1SubAuthority); 
  m_psidBuiltinDomainSid = (PSID)LocalAlloc(LPTR,dwSidWith1SubAuthority); 
 
  InitializeSid( m_psidNtAuthoritySid,   &siaNtAuthority, 0 ); 
  InitializeSid( m_psidDialupSid,        &siaNtAuthority, 1 ); 
  InitializeSid( m_psidNetworkSid,       &siaNtAuthority, 1 ); 
  InitializeSid( m_psidBatchSid,         &siaNtAuthority, 1 ); 
  InitializeSid( m_psidInteractiveSid,   &siaNtAuthority, 1 ); 
  InitializeSid( m_psidLogonIdsSid,      &siaNtAuthority, 3 ); 
  InitializeSid( m_psidServiceSid,       &siaNtAuthority, 1 ); 
  InitializeSid( m_psidLocalSystemSid,   &siaNtAuthority, 1 ); 
  InitializeSid( m_psidBuiltinDomainSid, &siaNtAuthority, 1 ); 
 
  *(GetSidSubAuthority( m_psidDialupSid,        0 )) = SECURITY_DIALUP_RID; 
  *(GetSidSubAuthority( m_psidNetworkSid,       0 )) = SECURITY_NETWORK_RID; 
  *(GetSidSubAuthority( m_psidBatchSid,         0 )) = SECURITY_BATCH_RID; 
  *(GetSidSubAuthority( m_psidInteractiveSid,   0 )) = SECURITY_INTERACTIVE_RID; 
  *(GetSidSubAuthority( m_psidLogonIdsSid,      0 )) = SECURITY_LOGON_IDS_RID; 
  *(GetSidSubAuthority( m_psidLogonIdsSid,      1 )) = 0; // Bogus! 
  *(GetSidSubAuthority( m_psidLogonIdsSid,      2 )) = 0; // Also bogus! 
  *(GetSidSubAuthority( m_psidServiceSid,       0 )) = SECURITY_SERVICE_RID; 
  *(GetSidSubAuthority( m_psidLocalSystemSid,   0 )) = SECURITY_LOCAL_SYSTEM_RID; 
  *(GetSidSubAuthority( m_psidBuiltinDomainSid, 0 )) = SECURITY_BUILTIN_DOMAIN_RID; 
} 

BOOL CFileStat::SIDStringName(PSID psidSID, LPTSTR lpszSIDStringName) 
{ 
 
  DWORD dwNumSubAuthorities; 
  DWORD dwLen; 
  DWORD dwSubAuthorityI; 
  #define BytesInIdentifierAuthority  6 
  SID_IDENTIFIER_AUTHORITY siaSidAuthority; 
  SID_IDENTIFIER_AUTHORITY siaNullSidAuthority    = SECURITY_NULL_SID_AUTHORITY; 
  SID_IDENTIFIER_AUTHORITY siaWorldSidAuthority   = SECURITY_WORLD_SID_AUTHORITY; 
  SID_IDENTIFIER_AUTHORITY siaLocalSidAuthority   = SECURITY_LOCAL_SID_AUTHORITY; 
  SID_IDENTIFIER_AUTHORITY siaCreatorSidAuthority = SECURITY_CREATOR_SID_AUTHORITY; 
  SID_IDENTIFIER_AUTHORITY siaNtAuthority         = SECURITY_NT_AUTHORITY; 
 
  dwLen = sprintf(lpszSIDStringName,"S-%d-",SID_REVISION); 
 
  if (SID_REVISION != ((PISID)psidSID)->Revision) 
  { dwLen += sprintf(lpszSIDStringName+dwLen,"bad_revision==%d",((PISID)psidSID)->Revision); 
  } 
 
  siaSidAuthority = *(GetSidIdentifierAuthority(psidSID)); 
 
  if      (0==memcmp(&siaSidAuthority,&siaNullSidAuthority   ,BytesInIdentifierAuthority)) 
  { dwLen += sprintf(lpszSIDStringName+dwLen,"0"); 
  } 
  else if (0==memcmp(&siaSidAuthority,&siaWorldSidAuthority  ,BytesInIdentifierAuthority)) 
  { dwLen += sprintf(lpszSIDStringName+dwLen,"1"); 
  } 
  else if (0==memcmp(&siaSidAuthority,&siaLocalSidAuthority  ,BytesInIdentifierAuthority)) 
  { dwLen += sprintf(lpszSIDStringName+dwLen,"2"); 
  } 
  else if (0==memcmp(&siaSidAuthority,&siaCreatorSidAuthority,BytesInIdentifierAuthority)) 
  { dwLen += sprintf(lpszSIDStringName+dwLen,"3"); 
  } 
  else if (0==memcmp(&siaSidAuthority,&siaNtAuthority        ,BytesInIdentifierAuthority)) 
  { dwLen += sprintf(lpszSIDStringName+dwLen,"5"); 
  } 
  else 
  { dwLen += sprintf(lpszSIDStringName+dwLen,"UnknownAuthority!"); 
  } 
 
  dwNumSubAuthorities = (DWORD)( *(GetSidSubAuthorityCount(psidSID)) ); 
 
  for (dwSubAuthorityI=0; dwSubAuthorityI<dwNumSubAuthorities; dwSubAuthorityI++) 
  { dwLen += sprintf(lpszSIDStringName+dwLen,"-%d",*(GetSidSubAuthority(psidSID,dwSubAuthorityI))); 
  } 
 
  return(TRUE); 
}

void CFileStat::EnableBCKprivs()
{	
	HANDLE hToken;
	BYTE byteBuff[sizeof TOKEN_PRIVILEGES* 2];
	TOKEN_PRIVILEGES& tkp = *((TOKEN_PRIVILEGES*)byteBuff);

	OpenProcessToken( GetCurrentProcess(),
		TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hToken); 
	// set up SeBackupPrivilege, SeRestorePrivilege
	LookupPrivilegeValue(NULL, SE_BACKUP_NAME, &tkp.Privileges[0].Luid);

	LookupPrivilegeValue(NULL, SE_RESTORE_NAME, &tkp.Privileges[1].Luid);

	tkp.PrivilegeCount = 2;
	tkp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
	tkp.Privileges[1].Attributes = SE_PRIVILEGE_ENABLED;
	AdjustTokenPrivileges(hToken, FALSE, &tkp, sizeof tkp, NULL, NULL);
}

int CFileStat::GetFileAttribs(CString fileName)
{
    m_strFileName = fileName;
	TCHAR* pFile = fileName.GetBuffer(500);

	FILETIME ftLocal;
    SYSTEMTIME stCreate;

	DWORD dwError = 0;
	DWORD dwAttribs = 0;
	int iRet;
	memset(&m_attrFile, NULL, sizeof(m_attrFile));  
	// store the file attributes for later user 
	iRet = GetFileAttributesEx(pFile, GetFileExInfoStandard, &m_attrFile); 
	if(iRet == 0)
	{
		dwError = GetLastError();
		return 0;
	}

	// Get the file times
	// Convert the creation time to local time.
    if (!FileTimeToLocalFileTime(&m_attrFile.ftCreationTime, &ftLocal)) 
		return FALSE;
    // Convert the local file time from UTC to system time.
    FileTimeToSystemTime(&ftLocal, &stCreate);
    // Build a string showing the date and time.
    m_strCreationTime.Format("%02d/%02d/%d  %02d:%02d:%02d",
        stCreate.wDay, stCreate.wMonth, stCreate.wYear,
        stCreate.wHour, stCreate.wMinute, stCreate.wSecond);

	// Convert the last-write time to local time.
    if (!FileTimeToLocalFileTime(&m_attrFile.ftLastWriteTime, &ftLocal))
		return FALSE;
    // Convert the local file time from UTC to system time.
    FileTimeToSystemTime(&ftLocal, &stCreate);
    // Build a string showing the date and time.
    m_strModifiedTime.Format("%02d/%02d/%d  %02d:%02d:%02d",
        stCreate.wDay, stCreate.wMonth, stCreate.wYear,
        stCreate.wHour, stCreate.wMinute, stCreate.wSecond);

	// Convert the last-access time to local time.
    if (!FileTimeToLocalFileTime(&m_attrFile.ftLastAccessTime, &ftLocal))
        return FALSE;
    // Convert the local file time from UTC to system time.
    FileTimeToSystemTime(&ftLocal, &stCreate);
    // Build a string showing the date and time.
    m_strAccessTime.Format("%02d/%02d/%d  %02d:%02d:%02d",
        stCreate.wDay, stCreate.wMonth, stCreate.wYear,
        stCreate.wHour, stCreate.wMinute, stCreate.wSecond);

	//Get file size, using only the low order bit here, in future use both for large files
    m_dwFileSize = m_attrFile.nFileSizeLow;

	//Grab and determine the file attributes
	m_dwFileAttributes = m_attrFile.dwFileAttributes;
	ParseAttribs(m_dwFileAttributes);

	//Grab the security attributes
	RecordSecStats();

	//Grab stream stats
	ListStreams();

   return true;
}

int CFileStat::GetFileTimes(CString& createTime, CString& modTime)
{
    //return ref to private time members 
	createTime = m_strCreationTime;
	modTime    = m_strModifiedTime;
  
	return true;
}

int CFileStat::PrintStats()
{
	cout << "\n";
    cout << "Creation Time - " <<  m_strCreationTime << "\n";
	cout << "Last Mod Time - " <<  m_strModifiedTime << "\n";
	cout << "Last Access Time - " <<  m_strAccessTime << "\n";
	cout << "Main File Size - " <<  m_dwFileSize << "\n";
	cout << "File Attrib Mask - ";
	if(m_bArchive)
		cout << "Arch ";
	if(m_bCompressed)
		cout << "Comp ";
	if(m_bDirectory)
		cout << "Dir ";
	if(m_bHidden) 		
	 	cout << "Hid ";	
	if(m_bNormal)
		cout << "Normal";
	if(m_bOffLine)
		cout << "Offline";
	if(m_bReadOnly)
		cout << "Read";
	if(m_bSystem)
		cout << "Sys";
	if(m_bTemp) 
		cout << "Temp"; 			
	 			
	return true;
}

int CFileStat::RecordSecStats()
{
	LPTSTR          lpszFullName = (LPTSTR)m_strFileName.GetBuffer(500); 
	#define              SZ_SD_BUF 8096 
	UCHAR                ucBuf[SZ_SD_BUF]; 
	DWORD                dwSDLength = SZ_SD_BUF; 
	DWORD                dwSDLengthNeeded = 0; 
	PSECURITY_DESCRIPTOR psdSD      = (PSECURITY_DESCRIPTOR)&ucBuf; 
	DWORD                dwError = 0; 
	int					 retVal = 0;

	retVal = GetFileSecurity(lpszFullName, 
							(SECURITY_INFORMATION)( OWNER_SECURITY_INFORMATION 
							   | GROUP_SECURITY_INFORMATION 
							   | DACL_SECURITY_INFORMATION 
							   | SACL_SECURITY_INFORMATION), 
					psdSD, 
					dwSDLength, 
					(LPDWORD)&dwSDLengthNeeded);
	 
	dwError = GetLastError();

	if(!ExamineSD(psdSD, FileAccessMask));
	{return FALSE;}

	dwError = GetLastError();

	return true;

}

void CFileStat::ParseAttribs(DWORD attribs)
{
	if(attribs & FILE_ATTRIBUTE_ARCHIVE)	//The file or directory is an archive file or directory. Applications use this attribute to mark files for backup or removal. 
	{m_bArchive = TRUE;}

	if(attribs & FILE_ATTRIBUTE_COMPRESSED)	//The file or directory is compressed. For a file, this means that all of the data in the file is compressed. For a directory, this means that compression is the default for newly created files and subdirectories. 
	{m_bCompressed = TRUE;}

	if(attribs & FILE_ATTRIBUTE_DIRECTORY)	//The handle identifies a directory. 
	{m_bDirectory = TRUE;}

	if(attribs & FILE_ATTRIBUTE_HIDDEN)		//The file or directory is hidden. It is not included in an ordinary directory listing. 
	{m_bHidden = TRUE;}

	if(attribs & FILE_ATTRIBUTE_NORMAL)		//The file or directory has no other attributes set. This attribute is valid only if used alone. 
	{m_bNormal = TRUE;}

	if(attribs & FILE_ATTRIBUTE_OFFLINE)	//The data of the file is not immediately available. Indicates that the file data has been physically moved to offline storage. 
	{m_bOffLine = TRUE;}

	if(attribs & FILE_ATTRIBUTE_READONLY)	//The file or directory is read-only. Applications can read the file but cannot write to it or delete it. In the case of a directory, applications cannot delete it. 
	{m_bReadOnly = TRUE;}

	if(attribs & FILE_ATTRIBUTE_SYSTEM)		//The file or directory is part of, or is used exclusively by, the operating system. 
	{m_bSystem = TRUE;}

	if(attribs & FILE_ATTRIBUTE_TEMPORARY)	//The file is being used for temporary storage. File systems attempt to keep all of the data in memory for quicker access rather than flushing the data back to mass storage. A temporary file should be deleted by the application as soon as it is no longer needed. 
	{m_bTemp = TRUE;}


}

BOOL CFileStat::ExamineSD(PSECURITY_DESCRIPTOR psdSD, 
                          KINDS_OF_ACCESSMASKS_DECODED kamKindOfMask) 
{ 
 
  PACL                        paclDACL; 
  PACL                        paclSACL; 
  BOOL                        bHasDACL        = FALSE; 
  BOOL                        bHasSACL        = FALSE; 
  BOOL                        bDaclDefaulted  = FALSE; 
  BOOL                        bSaclDefaulted  = FALSE; 
  BOOL                        bOwnerDefaulted = FALSE; 
  BOOL                        bGroupDefaulted = FALSE; 
  PSID                        psidOwner; 
  PSID                        psidGroup; 
  SECURITY_DESCRIPTOR_CONTROL sdcSDControl; 
  DWORD                       dwSDRevision; 
  DWORD                       dwSDLength; 
 
  if (!IsValidSecurityDescriptor(psdSD)) 
  {return FALSE;} 
 
  dwSDLength = GetSecurityDescriptorLength(psdSD); 
 
  if (!GetSecurityDescriptorDacl(psdSD, 
                                 (LPBOOL)&bHasDACL, 
                                 (PACL *)&paclDACL, 
                                 (LPBOOL)&bDaclDefaulted)) 
  {return FALSE;} 
 
  if (!GetSecurityDescriptorSacl(psdSD, 
                                 (LPBOOL)&bHasSACL, 
                                 (PACL *)&paclSACL, 
                                 (LPBOOL)&bSaclDefaulted)) 
  {return FALSE;} 
 
  if (!GetSecurityDescriptorOwner(psdSD, 
                                  (PSID *)&psidOwner, 
                                  (LPBOOL)&bOwnerDefaulted)) 
  {return FALSE;} 
 
  if (!GetSecurityDescriptorGroup(psdSD, 
                                  (PSID *)&psidGroup, 
                                  (LPBOOL)&bGroupDefaulted)) 
  {return FALSE;} 
 
  if (!GetSecurityDescriptorControl(psdSD, 
                                    (PSECURITY_DESCRIPTOR_CONTROL)&sdcSDControl, 
                                    (LPDWORD)&dwSDRevision)) 
  {return FALSE;} 
 
  printf("\nSD is valid.\nSD is %d bytes long.\nSD revision is %d == ", dwSDLength, dwSDRevision); 
 
  switch (dwSDRevision) 
  { 
    case      SECURITY_DESCRIPTOR_REVISION1 : 
    { printf("SECURITY_DESCRIPTOR_REVISION1"); 
      break; 
    } 
    default : 
    { printf("! SD Revision is unknown."); 
      return(FALSE); 
    } 
  } 
 
 
  if (NULL == psidOwner) 
  { printf("\nSD's Owner is NULL"); 
  } 
  else 
  { printf("\nSD's Owner is Not NULL"); 
    if (bOwnerDefaulted ) 
    { printf("\nSD's Owner-Defaulted flag is TRUE"); } 
    else 
    { printf("\nSD's Owner-Defaulted flag is FALSE"); } 
    if(!LookupSIDName(psidOwner,"")) 
    {  } 
  } 
 
  if (NULL == psidGroup) 
  { printf("\nSD's Group is NULL"); 
    printf("\nSD's Group being NULL is typical, GROUP in SD(s) is mainly for POSIX compliance"); 
  } 
  else 
  { 
	if (bGroupDefaulted) 
    { printf("\nSD's Group-Defaulted flag is TRUE"); } 
    else 
    { printf("\nSD's Group-Defaulted flag is FALSE"); } 
    if(!LookupSIDName(psidGroup,"")) 
    {  } 
  } 
 
  if(SE_DACL_PRESENT & sdcSDControl) 
  { printf("\nSD's DACL is Present"); 
    if (bDaclDefaulted) 
    { printf("\nSD's DACL-Defaulted flag is TRUE"); 
    } 
    else 
    { printf("\nSD's DACL-Defaulted flag is FALSE"); 
    } 
 
    if(NULL == paclDACL) 
    { printf("\nSD has a NULL DACL explicitly specified      (allows all access to Everyone)"); 
      printf("\n    This does not apply to this SD, but for comparison,"); 
      printf("\n    a non-NULL DACL pointer to a 0-length ACL allows  no access to   anyone"); 
    } 
    else if(!ExamineACL(paclDACL,"",kamKindOfMask)) 
    { } 
  } 
  else 
  { 
	printf("\nSD's DACL is Not Present"); 
    printf("\nSD has no DACL at all (Access = Everyone)"); 
  } 
 
  if   (SE_SACL_PRESENT & sdcSDControl) 
  { printf("\nSD's SACL is Present"); 
    if (bSaclDefaulted) 
    { printf("\nSD's SACL-Defaulted flag is TRUE"); } 
    else 
    { printf("\nSD's SACL-Defaulted flag is FALSE"); } 
 
    if (NULL == paclSACL) 
    { printf("\nSD has a NULL SACL explicitly specified"); } 

    else if(!ExamineACL(paclSACL,"",kamKindOfMask)) 
    { } 
  } 
  else 
  { printf("\nSD's SACL is Not Present"); } 

  return TRUE;
} 

BOOL CFileStat::ExamineACL(PACL paclACL, LPTSTR lpszOldIndent, 
							KINDS_OF_ACCESSMASKS_DECODED kamKindOfMask) 
{ 
  #define						SZ_INDENT_BUF 80 
  UCHAR							ucIndentBuf[SZ_INDENT_BUF] = ""; 
  ACL_SIZE_INFORMATION          asiAclSize; 
  ACL_REVISION_INFORMATION      ariAclRevision; 
  DWORD							dwBufLength; 
  DWORD							dwAcl_i; 
  ACCESS_ALLOWED_ACE*			paaAllowedAce; 
 
  strcpy((char*)ucIndentBuf, lpszOldIndent); 
  strcat((char*)ucIndentBuf,"    "); 
 
  if (!IsValidAcl(paclACL)) 
  {  
    return(FALSE); 
  } 
 
  dwBufLength = sizeof(asiAclSize); 
 
  if (!GetAclInformation(paclACL, 
                         (LPVOID)&asiAclSize, 
                         (DWORD)dwBufLength, 
                         (ACL_INFORMATION_CLASS)AclSizeInformation)) 
  { 
    return(FALSE); 
  } 
 
  dwBufLength = sizeof(ariAclRevision); 
 
  if (!GetAclInformation(paclACL, 
                         (LPVOID)&ariAclRevision, 
                         (DWORD)dwBufLength, 
                         (ACL_INFORMATION_CLASS)AclRevisionInformation)) 
  {  
    return(FALSE); 
  } 
 
  printf("\n%sACL has %d ACE(s), %d bytes used, %d bytes free",ucIndentBuf, 
     asiAclSize.AceCount, 
     asiAclSize.AclBytesInUse, 
     asiAclSize.AclBytesFree); 
 
  printf("\n%sACL revision is %d == ",ucIndentBuf,ariAclRevision.AclRevision); 
 
  switch (ariAclRevision.AclRevision) 
  { 
    case      ACL_REVISION1 : 
    { printf("ACL_REVISION1"); 
      break; 
    } 
    case      ACL_REVISION2 : 
    { printf("ACL_REVISION2"); 
      break; 
    } 
    default : 
    { printf("\n%sACL Revision unknown",ucIndentBuf); 
      return(FALSE); 
    } 
  } 
 
  for (dwAcl_i = 0; dwAcl_i < asiAclSize.AceCount;  dwAcl_i++) 
  { 
    if (!GetAce(paclACL, 
                dwAcl_i, 
                (LPVOID *)&paaAllowedAce)) 
    {  
      return FALSE; 
    } 

    if(!LookupSIDName((PSID)&(paaAllowedAce->SidStart),(char*)ucIndentBuf)) 
    { return FALSE; } 
 
	DWORD dwAceFlags = paaAllowedAce->Header.AceFlags; 
 
    switch (paaAllowedAce->Header.AceType) 
    { 
      case                       ACCESS_ALLOWED_ACE_TYPE : 
      { printf("\n%sACE %d is an ACCESS_ALLOWED_ACE_TYPE",ucIndentBuf,dwAcl_i); 
        break; 
      } 
      case                       ACCESS_DENIED_ACE_TYPE : 
      { printf("\n%sACE %d is an ACCESS_DENIED_ACE_TYPE",ucIndentBuf,dwAcl_i); 
        break; 
      } 
      case                       SYSTEM_AUDIT_ACE_TYPE : 
      { printf("\n%sACE %d is a SYSTEM_AUDIT_ACE_TYPE",ucIndentBuf,dwAcl_i); 
        break; 
      } 
      case                       SYSTEM_ALARM_ACE_TYPE : 
      { printf("\n%sACE %d is a SYSTEM_ALARM_ACE_TYPE",ucIndentBuf,dwAcl_i); 
        break; 
      } 
      default : 
      { printf("\n%sACE %d is an IMPOSSIBLE ACE_TYPE!!! Run debugger, examine value!",ucIndentBuf,dwAcl_i); 
        return FALSE; 
      } 
    } 

    printf("\n%sACE %d size = %d",ucIndentBuf,dwAcl_i,paaAllowedAce->Header.AceSize); 

    printf("\n%sACE %d flags = 0x%.2x",ucIndentBuf,dwAcl_i,dwAceFlags); 
 
      if (dwAceFlags) 
      { 
        DWORD dwExtraBits; 
        UCHAR ucIndentBitsBuf[SZ_INDENT_BUF] = ""; 
 
        strcpy((char*)ucIndentBitsBuf,(char*)ucIndentBuf); 
        strcat((char*)ucIndentBitsBuf,""); 
 
        if ((dwAceFlags  & OBJECT_INHERIT_ACE        ) == OBJECT_INHERIT_ACE        ) 
        { printf("\n%s0x01 OBJECT_INHERIT_ACE        ",ucIndentBitsBuf); 
        } 
        if ((dwAceFlags  & CONTAINER_INHERIT_ACE     ) == CONTAINER_INHERIT_ACE     ) 
        { printf("\n%s0x02 CONTAINER_INHERIT_ACE     ",ucIndentBitsBuf); 
        } 
        if ((dwAceFlags  & NO_PROPAGATE_INHERIT_ACE  ) == NO_PROPAGATE_INHERIT_ACE  ) 
        { printf("\n%s0x04 NO_PROPAGATE_INHERIT_ACE",ucIndentBitsBuf); 
        } 
        if ((dwAceFlags  & INHERIT_ONLY_ACE          ) == INHERIT_ONLY_ACE          ) 
        { printf("\n%s0x08 INHERIT_ONLY_ACE",ucIndentBitsBuf); 
        } 
        if ((dwAceFlags  & VALID_INHERIT_FLAGS       ) == VALID_INHERIT_FLAGS       ) 
        { printf("\n%s0x0F VALID_INHERIT_FLAGS",ucIndentBitsBuf); 
        } 
        if ((dwAceFlags  & SUCCESSFUL_ACCESS_ACE_FLAG) == SUCCESSFUL_ACCESS_ACE_FLAG) 
        { 
			printf("\n%sACE %d audit type = SUCCESSFUL_ACCESS_ACE",ucIndentBitsBuf, dwAcl_i);
        } 
        if ((dwAceFlags  & FAILED_ACCESS_ACE_FLAG    ) == FAILED_ACCESS_ACE_FLAG) 
        { 
			printf("\n%sACE %d audit type = FAILED_ACCESS_ACE",ucIndentBitsBuf, dwAcl_i); 
        } 
 
        dwExtraBits = dwAceFlags & ( ~( OBJECT_INHERIT_ACE 
                                      | CONTAINER_INHERIT_ACE 
                                      | NO_PROPAGATE_INHERIT_ACE 
                                      | INHERIT_ONLY_ACE 
                                      | VALID_INHERIT_FLAGS 
                                      | SUCCESSFUL_ACCESS_ACE_FLAG 
                                      | FAILED_ACCESS_ACE_FLAG) ); 
        if (dwExtraBits) 
        {
			printf("\n%sExtra AceFlag bits     == 0x%.8x <-This is a problem, should be all 0s",ucIndentBuf,dwExtraBits); 
        } 
      } 
 
    printf("\n%sACE %d mask = 0x%.8x",ucIndentBuf,dwAcl_i,paaAllowedAce->Mask); 
	//break up the mask and show what perms are really there
	ExamineMask(paaAllowedAce->Mask,kamKindOfMask);

  } 

  return TRUE;
} 

BOOL CFileStat::LookupSIDName(PSID psidSID, LPTSTR lpszOldIndent) 
{ 
  UCHAR        ucIndentBuf    [SZ_INDENT_BUF]    = ""; 
  #define                      SZ_ACCT_NAME_BUF  60 
  UCHAR        ucNameBuf      [SZ_ACCT_NAME_BUF] = ""; 
  DWORD        dwNameLength  = SZ_ACCT_NAME_BUF; 
  #define                      SZ_DMN_NAME_BUF   60 
  UCHAR        ucDomainNmBuf  [SZ_DMN_NAME_BUF]  = ""; 
  DWORD        dwDNameLength = SZ_DMN_NAME_BUF; 
  #define                      SZ_SID_STRING_BUF 150 
  UCHAR        ucSIDStringBuf [SZ_SID_STRING_BUF] = ""; 
  SID_NAME_USE peAcctNameUse = SidTypeInvalid; 
  DWORD        dwLookupStatus; 
  BOOL         bGotBadLookupThatIsNotLocalLogonSID; 
 
  strcpy((char*)ucIndentBuf,lpszOldIndent); 
  strcat((char*)ucIndentBuf,"  "); 
 
  if (!IsValidSid(psidSID)) 
  { 
    return(FALSE); 
  } 
 
  if (!SIDStringName(psidSID, (char*)ucSIDStringBuf)) 
  {
    return FALSE; 
  } 
 
  if (!LookupAccountSid( 
         (LPTSTR)"",         // Look on local machine 
         psidSID, 
         (LPTSTR)&ucNameBuf, 
         (LPDWORD)&dwNameLength, 
         (LPTSTR)&ucDomainNmBuf, 
         (LPDWORD)&dwDNameLength, 
         (PSID_NAME_USE)&peAcctNameUse)) 
  { 
    dwLookupStatus = GetLastError(); 
 
    bGotBadLookupThatIsNotLocalLogonSID = FALSE;  
 
    if ( ( *(GetSidSubAuthorityCount(m_psidLogonIdsSid))) != 
         ( *(GetSidSubAuthorityCount(psidSID        )))    ) 
    { // Not same number of sub-authorities, so can't be a match 
      bGotBadLookupThatIsNotLocalLogonSID = TRUE; 
    } 
    else 
    { 
      // Force the last two sub-authorities to match 
      *(GetSidSubAuthority( m_psidLogonIdsSid, 1 )) = 
      *(GetSidSubAuthority( psidSID        , 1 )); 
      *(GetSidSubAuthority( m_psidLogonIdsSid, 2 )) = 
      *(GetSidSubAuthority( psidSID        , 2 )); 
 

      if (EqualSid(psidSID, m_psidLogonIdsSid)) 
      { printf("\n%sSID is the Local Logon SID   %s",ucIndentBuf,ucSIDStringBuf); 
      } 
      else 
      { bGotBadLookupThatIsNotLocalLogonSID = TRUE; 
      } 
	} 
    if (bGotBadLookupThatIsNotLocalLogonSID) 
    { 
 
      if (ERROR_NONE_MAPPED == dwLookupStatus) 
      { 
		  printf("\n  SID = %s/%s    (Account Unknown)  %s",ucDomainNmBuf,ucNameBuf,ucSIDStringBuf); 
      } 
      else 
      {
        return FALSE; 
      } 
    } 
  } 
	else 
	{ // Got good Lookup, so SID Is NOT the Local Logon SID 
		printf("\n  SID = %s/%s   %s",ucDomainNmBuf,ucNameBuf,ucSIDStringBuf); 
    } 

  return TRUE;
} 

void CFileStat::DumpHeader(WIN32_STREAM_ID& streamID, int iCount)
{
	const char* pStreamName;
	printf("\nStream %d:\n", iCount);
	switch (streamID.dwStreamId)
	{
	case BACKUP_DATA:	
		pStreamName = "Data";	
		break;
	case BACKUP_EA_DATA:
		pStreamName = "Extended Attributes";	
		break;	
	case BACKUP_SECURITY_DATA:	
		pStreamName = "Security";
		break;
	case BACKUP_ALTERNATE_DATA:	
		pStreamName = "Alternate Stream";
		break;
	case BACKUP_LINK:
		pStreamName = "Link";
		break;
	default:
		pStreamName = "Unknown";
		break;
	}

	char hiddenName[500];
	// Convert the Unicode full name to ANSI.
	WideCharToMultiByte( CP_ACP, 0, streamID.cStreamName, -1,
						hiddenName, 500, NULL, NULL ); 

	char seps[] = ":";;
	char* token;
	token = strtok(hiddenName, seps);
	printf("  Type: %s\n", pStreamName);
	printf("  Stream name = %s Size: %I64d\n", token, streamID.Size.QuadPart);
}

int CFileStat::ListStreams()
{
	//thanks to felix for the hard part, and the even better zw method

  	HANDLE hFile;	
	LPCTSTR lpFileName = (LPCTSTR)m_strFileName.GetBuffer(500);

	hFile = CreateFile(lpFileName, GENERIC_READ, FILE_SHARE_WRITE, 0, OPEN_EXISTING, 0, 0);

	if (hFile == INVALID_HANDLE_VALUE || hFile == NULL)	
	{ 
	    DWORD error = GetLastError();	
		return FALSE; 
	}
	
	BYTE byteBuff[4096];
	DWORD dwRead;
	void* pContext = NULL;
	WIN32_STREAM_ID& streamID = *((WIN32_STREAM_ID*)byteBuff);


	int iCount = 0;
	while (1)
	{	// grab stream header
		if (!BackupRead(hFile, byteBuff, 20, &dwRead, FALSE, TRUE, &pContext))	
		{
		
		}
		if (dwRead == 0 )
			break;
		if (streamID.dwStreamNameSize > 0)	
		{
			if (!BackupRead(hFile, byteBuff + 20, streamID.dwStreamNameSize, &dwRead, FALSE, TRUE, &pContext))
			{ 
			
			}
			if (dwRead != streamID.dwStreamNameSize)
				break;
		}	
		
		iCount++;
		DumpHeader(streamID, iCount);
		// skip stream data	
		if ( streamID.Size.QuadPart > 0 )
		{	
			DWORD dwLo, dwHi;
			BackupSeek( hFile, 0xffffffffL, 0x7fffffffL, &dwLo, &dwHi, &pContext);	
		}
	}

	//Release 
	BackupRead(hFile, byteBuff, 0, &dwRead, TRUE, FALSE, &pContext);
	CloseHandle(hFile);
	return 0;

  return TRUE;
}


void CFileStat::ExamineMask(ACCESS_MASK amMask, KINDS_OF_ACCESSMASKS_DECODED kamKindOfMask)
{
  #define STANDARD_RIGHTS_ALL_THE_BITS 0x00FF0000L
  #define GENERIC_RIGHTS_ALL_THE_BITS  0xF0000000L
  DWORD dwGenericBits;
  DWORD dwStandardBits;
  DWORD dwSpecificBits;
  DWORD dwAccessSystemSecurityBit;

  dwStandardBits            = (amMask); //& STANDARD_RIGHTS_ALL_THE_BITS);
  dwSpecificBits            = (amMask); //& SPECIFIC_RIGHTS_ALL         );
  dwAccessSystemSecurityBit = (amMask); //& ACCESS_SYSTEM_SECURITY      );
  dwGenericBits             = (amMask);// & GENERIC_RIGHTS_ALL_THE_BITS );

  //Print then decode the specific rights bits

  if (dwSpecificBits)
  {
    if(FileAccessMask == kamKindOfMask)
    {

      if (((dwStandardBits | dwSpecificBits   )
           & FILE_GENERIC_READ   ) == FILE_GENERIC_READ   )
      { 
		  printf(" %s", "-R");
      }
      if (((dwStandardBits | dwSpecificBits   )
           & FILE_GENERIC_WRITE  ) == FILE_GENERIC_WRITE  )
      { 
  		  printf(" %s", "-W");
      }
      if (((dwStandardBits | dwSpecificBits   )
           & FILE_GENERIC_EXECUTE) == FILE_GENERIC_EXECUTE)
      { 
  		  printf(" %s", "-X");
      }
		if ((dwStandardBits    & DELETE  ) == DELETE  )
		{ 
  			printf(" %s", "-D");
		}
      if ((dwSpecificBits    & FILE_DELETE_CHILD ) == FILE_DELETE_CHILD  )
      { 
			printf(" %s", "-DEL_CHILD");
      }
		if ((dwStandardBits    & WRITE_DAC  ) == WRITE_DAC )
		{ 
  			printf(" %s", "-CHANGE_PERMS");
		}
		if ((dwStandardBits    & WRITE_OWNER ) == WRITE_OWNER )
		{ 
  			printf(" %s", "-TAKE_OWN");
		}

    }
  }

}
