//HFind v1.0 - Copyright(c) 1998 by JD Glaser - NT OBJECTives, Inc. ALL RIGHTS RESERVED
//NTFS hidden file finder

 ********************************************************************
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 1, or (at your option)
 *  any later version. *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details. *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

#include <windows.h>
#include <tchar.h>
#include <iomanip.h>
#include <stdio.h>

const int MAXPATH = 500;

void ScanDir(bool bScanSubs);
void PrintHelp();

void main(int argc, char* argv[])
{
	bool bScanSubs = true;
	char CurDir[MAXPATH];

	// check if running on Windows NT, if not, display notice and terminate 
    if( GetVersion() & 0x80000000 ) 
    { 
      printf("HFind only runs on Windows NT...");
      return; 
    }

try
{
	//set thread priority to lowest
	HANDLE hThread;
	hThread = ::GetCurrentThread();
	::SetThreadPriority(hThread, THREAD_PRIORITY_LOWEST);

	if(argc > 1)
	{
		for (int ctr = 1; ctr < argc; ctr++)
		{
			const char* pArg = argv[ctr];

			if((0 == strcmp(pArg, "-ns")) || (0 == strcmp(pArg, "/ns")))
			{
				bScanSubs = false;
			}
			
			else if((0 == strcmp(pArg, "-?")) || (0 == strcmp(pArg, "/?")))
			{
				PrintHelp();
				return;
			}

			else
			{
				char* pNewPath = argv[ctr];
				::SetCurrentDirectory(pNewPath);
			}
		}
	}
	else
	{
		::GetCurrentDirectory(sizeof(CurDir), CurDir);
	}	

	//signal the start
	printf("Searching...\n");
	//Start the recursive search
	ScanDir(bScanSubs);

	//reset to original starting point
	::SetCurrentDirectory(CurDir);

	//print a closing notice
	char* pEndNotice = "Finished";
	printf("\r%-80s", pEndNotice);
	return;
}

	catch(...)
	{
		printf("\nError - Shutting Down...");
		return;
	}

}

void ScanDir(bool bScanSubs)
{
	bool bPrintOnce = true;

	char CurDir[MAXPATH];
	::GetCurrentDirectory(sizeof(CurDir), CurDir);
	CurDir[79] = '\0';
	printf("\r%-79s", CurDir);

	WIN32_FIND_DATA data;
	HANDLE hFile = ::FindFirstFile("*.*", &data);
	while(FindNextFile(hFile, &data))
	{
		if(data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY
			&& data.cFileName[0] != '.' && bScanSubs)
		{
			::SetCurrentDirectory(data.cFileName);
			bPrintOnce = true;
			ScanDir(bScanSubs);
			::SetCurrentDirectory("..");
		}
	
		else if(data.dwFileAttributes & FILE_ATTRIBUTE_HIDDEN)//&& (data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
		{
			FILETIME ftLocal;
			SYSTEMTIME stLastAccess;

			//don't print if parent dir 
			char* pTest = data.cFileName;
			if((0 == strcmp(pTest, "..")))
				continue;

			//get the last-access time to local time.
			if (!FileTimeToLocalFileTime(&data.ftLastAccessTime, &ftLocal)) 
				return;
			// Convert the local file time from UTC to system time.
			FileTimeToSystemTime(&ftLocal, &stLastAccess);
			//group files by directory
			if(bPrintOnce)
			{
				char CurDir[MAXPATH];
				::GetCurrentDirectory(sizeof(CurDir), CurDir);
				printf("\r%s\n", CurDir);
				bPrintOnce = false;
			}
			// Build a string showing the date and time.
			printf("  %s\t\t\t\t%02d/%02d/%d %02d:%02d:%02d\n",
						data.cFileName,
						stLastAccess.wDay, stLastAccess.wMonth, stLastAccess.wYear,
						stLastAccess.wHour, stLastAccess.wMinute, stLastAccess.wSecond);
			
		}
	}

	::FindClose(hFile);
};

void PrintHelp()
{
	cout << "HFind v1.2 - Copyright(c) 1998, NT OBJECTives, Inc." << endl;
	cout << "Hidden file finder with last access times" << endl;
	cout << "Programming by JD Glaser - All Rights Reserved" << endl;
	cout << "\t" << "Usage - hfind [path] /ns" << endl;
	cout << "\t" << "[dirpath]" << "\t" << "Directory to search - none equals current" << endl;
	cout << "\t" << "-ns" << "\t\t" << "Skip sub-directories" << endl;
	cout << "\t" << "- or /" << "\t\t" << "Either switch statement can be used" << endl;
	cout << "\t" << "-?" << "\t\t" << "Help" << endl;
	cout << "COMMAND PROMPT MUST HAVE A MINIMUM WIDTH OF 80 CHARACTERS" << endl;
	cout << "See http://www.ntobjectives.com for updates/fixes" << endl;
	return;
}
